/* Turn wiki-style formatting into HTML formatting.
   Document structuring templates to come later
*/  

wikiData = ''

function wikiTemplate(name) {return new Stream('file://' + wikiData + name + 
'.template','rt');}

function wikiPages()
{
 var files = system.files('*.txt')
 for (var i in files) files[i] = files[i].substr(0,files[i].length-4)
 return files
}

function WikiFile(name)
{
 this.name = name.replace(/\.txt$/,'')
 this.file = this.name.replace(/[\/\\]/g,'')
 if (!system.exists(wikiData+this.file+'.txt')) throw 'A page for "' + name + '" does not exist'
}

WikiFile.prototype.read = function()
{
 s = new Stream('file://' + wikiData + this.file + '.txt','rt')
 return [s.readMIME().toObject(), s]
}

WikiFile.prototype.write = function(header,text)
{
 var s = new Stream('file://' + wikiData + this.file + '.txt','wt');
 s.writeMIME(header)
 s.write(text)
 s.close() 
}

WikiFile.prototype.header = function()
{
 var [h, s] = this.read()
 s.close()
 return h
}

WikiFile.prototype.backup = function(date)
{
 var backup = wikiData + this.file + '.' + date.toLocaleFormat("%Y%m%d%H%M%S") + '.backup'
 if (!system.exists(wikiData + this.file + '.txt')) return false; 
 if (system.copy(wikiData + this.file + '.txt', backup))
 {
  system.execute('zip','-m "' + wikiData+ 'backup.zip" "' + wikiData + backup + '"')
  return true;
 }
 return false;
}

function WikiHistory(name)
{
 this.name = name.replace(/\.txt$/,'')
 this.file = this.name.replace(/[\/\\]/g,'')
 if (!this.file) throw ('Invalid name: "' + name + '"')
 
 this.files = system.files(this.file + "*.backup") 
 if (!this.files) this.files = []
 if (system.exists('backup.zip'))
 {
  var zip = new Archive('backup.zip')
  var s = this.file + '.'
  for (let i = 0; i< zip.count; i++)
  {
   let n = zip.name(i)
   if (n.indexOf(s) == 0)
    this.files.push(n)
  }
  zip.close()
 }
}

WikiHistory.prototype.read = function(name)
{
 var s = null
 if (system.exists(name))
   s = new Stream('file://' + wikiData + name, 'rt')
 else if (system.exists('backup.zip'))
 {
  var zip = new Archive('backup.zip')
  s = zip.extract(name)
  zip.close()
 }
 return [s.readMIME().toObject(), s]
}

function wikiStart()
{
  if (!system.exists("page.template"))
  {
   let s = new Stream('file://page.template','wt')
   s.writeln('<html><head><title>{name}</title></head><body>{toc}<hr>{contents}</body></html>')
   s.close()
  }
  if (!system.exists("Index.txt"))
  {
   let s = wikiTextSave('Index')
   s.writeln("Template: page")
   s.writeln()
   s.close()
  }
}

function wikiUpdate(name,text,template,extra)
{
var page = new WikiFile(name)
var header = page.header()
var now = new Date()

if (!page.backup(now)) throw "Backup failed for " + name

header['Date'] = now.toString() //toLocaleFormat("%Y-%m-%d %H:%M:%S")
if (template) header['Template']=template
for (var i in extra) header[i]=extra[i]
page.write(header,text)
wikify(page.file)
}

function wiki2html(text,links)
{
 if (text.constructor.name == 'Stream')
   text = text.readFile()
 if (!links) links = []
 var lines = text.split('\n')
 var last = ""
 var out = ""
 var intable = false
 var endtag = ''
 var toc = ''
 var numbers = [0,0,0,0,0]

 for each (let l in lines) 
 {
  if (!l.length)
  {
   if (endtag) out += endtag + "\n"
   endtag = ''
   last = l
   continue
  }
  
  //link formatting
  l = l.replace(/[^\[](http:\/\/[a-zA-Z0-9+&\/?=.]+)/g,'<a href="$1">$1</a>')
  l = l.replace(/\[\[?(.*?)\]\]?/g,function (a,l) 
   {
    if (l.indexOf("://") != -1)
    {
     let d = Math.max(l.indexOf(' '),l.indexOf('|'))
     return('<a href="' + l.substr(0,d) + '">' + l.substr(d+1) + '</a>')
    }
    else
    {
     links.push(l); return '<a href="' + encodeURL(l) + '.html">' + l + '</a>'
    }
   })

  //forbidden tags
  l = l.replace(/(<script>|<iframe>|<object>|<embed>)/g,'<tt>')
  l = l.replace(/(<\/script>|<\/iframe>|<\/object>|<\/embed>)/g,'</tt>')
    
  //event handlers
  if (l.search(/<[^>]*on\w+=/) != -1)
    l = encodeHTML(l)

  if (l[0] == '=') //new section or subsection
  {
   if (endtag) out += endtag + "\n"
   endtag = ''
   let level = 1
   while (l[0] == '=')
    {
     level++
     l = l.substr(1)   
    }

   l = l.replace(/=+$/,'')

   let n = ''
   if (level <= numbers.length)
   {
    numbers[level-1]++
    for (let i=level; i<numbers.length; i++) numbers[i] = 0;
    for (let i=1; i < level ; i++) n += numbers[i] + '.';
    for (let i=2; i < level ; i++) toc += '&nbsp;&nbsp;&nbsp;&nbsp;'

    if (n) 
    {
     out += '<a name="a' + n + '" />';
     toc += '<a class="toc" href="#a' + n + '">';
     n += ' '
     toc += n + l.replace(/\@.*/g,'') + '</a><br>'
    }
   }

   out += "<h" + level + ">" + n + l + "</h" + level + ">\n"   
//   endtag = "</p>"
  }
  else if (l[0] == ' ') //quotation
  {
   if (last.length && last[0] != ' ')
   {
    if (endtag) out += endtag
    out += "<p class=quote>"
    endtag = "</p>"
   }
   out += l + "\n"
  }
  else if (l[0] == '*') //new list
  {
   if (endtag != "</ul>")
   { 
    if (endtag) out += endtag
    out += "<ul>"
    endtag = "</ul>"
   }
   out += "<li>" + l.substr(1) + "</li>\n"
  }  
  else if (l[0] == '#') //new list
  {
   if (endtag != "</ol>")
   { 
    if (endtag) out += endtag
    out += "<ol>"
    endtag = "</ol>"
   }
   out += "<li>" + l.substr(1) + "</li>\n"
  }  
  else if (l.indexOf('\t') != -1) //table
  {
    if (endtag != "</table>") 
    {
    if (endtag) out += endtag
      out += "<table>\n"
      endtag = "</table>"
    }
    out += "<tr><td>" + l.replace(/\t/g,"</td><td>") + "</td></tr>"    
  }
  else if (!last.length) //new paragraph
  {  
    if (endtag) out += endtag
   out += "<p>" + l
   endtag = "</p>"
  }
  else 
  {    
    out += l + "\n"
  }  
  
  last = l
 }
 
 out += endtag + "\n"
 
 return [out,toc]
}

function wikify(files)
{
  if (typeof files == "string") files = [files]
  var links = []
  var pages = []
  var lastTemplate = ''
  var lines
  var now = new Date()
   
  for (let i=0; i< files.length; i++)
  {
   var p ;   
   try {
   p = new WikiFile(files[i])
   } 
   catch(x) {continue;}

   var [d, s] = p.read();
   
   var meta = ''
   for (var n in d)
    meta += "<b>" + n + "</b>: " + d[n] + "<br>"

   pages.push({name: p.name, header: d, meta: meta, text: s.readFile()})
   s.close()

   for each (let a in pages)
   {
    let out = new Stream(a.name + '.html', 'wt')
    var templFile = a.header['Template']
    if (!templFile) templFile = 'page'
    if (templFile != lastTemplate)
    {
      var template = wikiTemplate(templFile)
      lines = template.readFile().split('\n')
      template.close()
    }

    var [contents,toc] =  wiki2html(a.text,links)

    for each (let l in lines)
    {
      out.writeln(l.replace(/{([\w\.]+)}/g,
        function(x,m)
        { switch(m)
          { case 'contents': return contents;
            case 'toc': return toc;
            case 'meta': return a.meta;
            case 'name': return a.name;
            default: return a.header[m]
           }
        }
      ))
    }

    out.close()
   }
   
   for each (let l in links)
   {
    if (!system.exists(l + ".txt"))
    {
      let s = new Stream(l + ".txt",'wt')
      s.writeln("Author: ")
      s.writeln("Date: ", now.toLocaleFormat("%Y/%m/%d %H:%M:%S"))
      s.writeln("Location: ")
      s.writeln("Template: page")
      s.writeln()
      s.writeln("="+l)
      s.writeln()
      s.close()
      files.push(l + ".txt")
    }
   }
  }
}
